@@ -194,7 +194,7 @@ namespace Ogre
 
     void ScriptCompilerListener::preConversion(ScriptCompiler *compiler, ConcreteNodeListPtr nodes)
     {
-        
+
     }
 
     bool ScriptCompilerListener::postConversion(ScriptCompiler *compiler, const AbstractNodeListPtr &nodes)
@@ -208,7 +208,7 @@ namespace Ogre
         ss << "ScriptCompiler - " << ScriptCompiler::formatErrorCode(code) << " in " << file << "(" << line << ")";
         if(!msg.empty())
             ss << ": " << msg;
-        
+
         if(code == ScriptCompiler::CE_DEPRECATEDSYMBOL)
             LogManager::getSingleton().logWarning(ss.str());
         else
@@ -287,10 +287,10 @@ namespace Ogre
             {
                 PropertyAbstractNode *prop = static_cast<PropertyAbstractNode*>(node.get());
                 msg = msg + prop->name + " =";
-                for(AbstractNodeList::iterator i = prop->values.begin(); i != prop->values.end(); ++i)
+                for(const auto& v : prop->values)
                 {
-                    if((*i)->type == ANT_ATOM)
-                        msg = msg + " " + static_cast<AtomAbstractNode*>((*i).get())->value;
+                    if(v->type == ANT_ATOM)
+                        msg = msg + " " + static_cast<AtomAbstractNode*>(v.get())->value;
                 }
             }
             break;
@@ -298,10 +298,10 @@ namespace Ogre
             {
                 ObjectAbstractNode *obj = static_cast<ObjectAbstractNode*>(node.get());
                 msg = msg + node->file + " - " + StringConverter::toString(node->line) + " - " + obj->cls + " \"" + obj->name + "\" =";
-                for(AbstractNodeList::iterator i = obj->values.begin(); i != obj->values.end(); ++i)
+                for(const auto& v : obj->values)
                 {
-                    if((*i)->type == ANT_ATOM)
-                        msg = msg + " " + static_cast<AtomAbstractNode*>((*i).get())->value;
+                    if(v->type == ANT_ATOM)
+                        msg = msg + " " + static_cast<AtomAbstractNode*>(v.get())->value;
                 }
             }
             break;
@@ -314,9 +314,9 @@ namespace Ogre
         if(node->type == ANT_OBJECT)
         {
             ObjectAbstractNode *obj = static_cast<ObjectAbstractNode*>(node.get());
-            for(AbstractNodeList::iterator i = obj->children.begin(); i != obj->children.end(); ++i)
+            for(const auto& c : obj->children)
             {
-                logAST(tabs + 1, *i);
+                logAST(tabs + 1, c);
             }
         }
     }
@@ -324,6 +324,11 @@ namespace Ogre
 
     bool ScriptCompiler::compile(const ConcreteNodeListPtr &nodes, const String &group)
     {
+        if(nodes && !nodes->empty())
+        {
+            mSourceFile = nodes->front()->file;
+        }
+
         // Set up the compilation context
         mGroup = group;
 
@@ -348,12 +353,12 @@ namespace Ogre
         // Allows early bail-out through the listener
         if(mListener && !mListener->postConversion(this, ast))
             return mErrors.empty();
-        
+
         // Translate the nodes
         for(auto & i : *ast)
         {
 #if DEBUG_AST
-            logAST(0, *i);
+            logAST(0, i);
 #endif
             if(i->type == ANT_OBJECT && static_cast<ObjectAbstractNode*>(i.get())->abstract)
                 continue;
@@ -366,6 +371,7 @@ namespace Ogre
         mImports.clear();
         mImportRequests.clear();
         mImportTable.clear();
+        mSourceFile.clear();
 
         return mErrors.empty();
     }
@@ -385,6 +391,11 @@ namespace Ogre
         mErrors.push_back({file, msg, line, code});
     }
 
+    void ScriptCompiler::addError(const AbstractNode& node, const String& msg, uint32 code)
+    {
+        addError(code, mSourceFile, node.line, msg);
+    }
+
     void ScriptCompiler::setListener(ScriptCompilerListener *listener)
     {
         mListener = listener;
@@ -528,7 +539,7 @@ namespace Ogre
     AbstractNodeList ScriptCompiler::locateTarget(const AbstractNodeList& nodes, const Ogre::String &target)
     {
         SharedPtr<AbstractNode> iter = nullptr;
-    
+
         // Search for a top-level object node
         for(auto i : nodes)
         {
@@ -1045,7 +1056,9 @@ namespace Ogre
         mIds["3d"] = ID_3D;
         mIds["cubic"] = ID_CUBIC;
         mIds["unlimited"] = ID_UNLIMITED;
-        mIds["2darray"] = ID_2DARRAY;
+        mIds["2darray"] = ID_2DARRAY; // deprecated and undocumented
+        mIds["2d_array"] = ID_2DARRAY;
+        mIds["2d_ms"] = ID_2DMS;
         mIds["alpha"] = ID_ALPHA;
         mIds["gamma"] = ID_GAMMA;
         mIds["anim_texture"] = ID_ANIM_TEXTURE;
@@ -1127,6 +1140,7 @@ namespace Ogre
         mIds["shared_params"] = ID_SHARED_PARAMS;
         mIds["shared_param_named"] = ID_SHARED_PARAM_NAMED;
         mIds["shared_params_ref"] = ID_SHARED_PARAMS_REF;
+        mIds["use_linear_colours"] = ID_USE_LINEAR_COLOURS;
 
         // Particle system
         mIds["particle_system"] = ID_PARTICLE_SYSTEM;
@@ -1156,7 +1170,7 @@ namespace Ogre
         mIds["chain_scope"] = ID_SCOPE_CHAIN;
         mIds["global_scope"] = ID_SCOPE_GLOBAL;
         mIds["compositor_logic"] = ID_COMPOSITOR_LOGIC;
-            
+
         mIds["only_initial"] = ID_ONLY_INITIAL;
         mIds["visibility_mask"] = ID_VISIBILITY_MASK;
         mIds["lod_bias"] = ID_LOD_BIAS;
@@ -1256,7 +1270,7 @@ namespace Ogre
             ImportAbstractNode *impl = OGRE_NEW ImportAbstractNode();
             impl->line = node->line;
             impl->file = node->file;
-            
+
             ConcreteNodeList::iterator iter = node->children.begin();
             impl->target = (*iter)->token;
 
@@ -1350,14 +1364,14 @@ namespace Ogre
                 if(node->token == "abstract")
                 {
                     impl->abstract = true;
-                    for(ConcreteNodeList::const_iterator i = node->children.begin(); i != node->children.end(); ++i)
-                        temp.push_back((*i).get());
+                    for(const auto& c : node->children)
+                        temp.push_back(c.get());
                 }
                 else
                 {
                     temp.push_back(node);
-                    for(ConcreteNodeList::const_iterator i = node->children.begin(); i != node->children.end(); ++i)
-                        temp.push_back((*i).get());
+                    for(const auto& c : node->children)
+                        temp.push_back(c.get());
                 }
 
                 // Get the type of object
@@ -1468,6 +1482,12 @@ namespace Ogre
                 impl->id = iter2->second;
 
             asn = AbstractNodePtr(impl);
+
+            if(mCurrent && mCurrent->type != ANT_PROPERTY)
+            {
+                // stray atom outside of a property is likely a property without a value
+                mCompiler->addError(*impl, "missing property value");
+            }
         }
 
         // Here, we must insert the node into the tree
@@ -1498,19 +1518,19 @@ namespace Ogre
         for(const auto & node : nodes)
             visitor->visit(node.get());
     }
-    
+
 
     // ScriptCompilerManager
     template<> ScriptCompilerManager *Singleton<ScriptCompilerManager>::msSingleton = 0;
-    
+
     ScriptCompilerManager* ScriptCompilerManager::getSingletonPtr(void)
     {
         return msSingleton;
     }
     //-----------------------------------------------------------------------
     ScriptCompilerManager& ScriptCompilerManager::getSingleton(void)
-    {  
-        assert( msSingleton );  return ( *msSingleton );  
+    {
+        assert( msSingleton );  return ( *msSingleton );
     }
     //-----------------------------------------------------------------------
     ScriptCompilerManager::ScriptCompilerManager()
@@ -1552,7 +1572,7 @@ namespace Ogre
     void ScriptCompilerManager::removeTranslatorManager(Ogre::ScriptTranslatorManager *man)
     {
             OGRE_LOCK_AUTO_MUTEX;
-        
+
         for(std::vector<ScriptTranslatorManager*>::iterator i = mManagers.begin(); i != mManagers.end(); ++i)
         {
             if(*i == man)
@@ -1573,7 +1593,7 @@ namespace Ogre
         ScriptTranslator *translator = 0;
         {
                     OGRE_LOCK_AUTO_MUTEX;
-            
+
             // Start looking from the back
             for(std::vector<ScriptTranslatorManager*>::reverse_iterator i = mManagers.rbegin(); i != mManagers.rend(); ++i)
             {
