@@ -53,7 +53,8 @@ namespace Ogre {
 
 		for (i = mProfiles.begin(); i != iend; ++i)
 		{
-			bool syntaxSupported = gpuMgr.isSyntaxSupported(*i);
+			String profile = *i == "ps_2_x" ? "ps_2_a" : *i;
+			bool syntaxSupported = gpuMgr.isSyntaxSupported(profile);
 			if (!syntaxSupported && find(specialCgProfiles, specialCgProfilesEnd, *i) != specialCgProfilesEnd)
 			{
 				// Cg has some "special" profiles which don't have direct equivalents
@@ -69,8 +70,8 @@ namespace Ogre {
 
 			if (syntaxSupported)
 			{
-				mSelectedProfile = *i;
-				String selectedProfileForFind = mSelectedProfile;
+				mSelectedProfile = profile;
+				String selectedProfileForFind = *i;
 				if(StringUtil::startsWith(mSelectedProfile,"vs_4_0_", true))
 				{
 					selectedProfileForFind = "vs_4_0";
@@ -115,7 +116,13 @@ namespace Ogre {
 		if (!mPreprocessorDefines.empty())
 			args = StringUtil::split(mPreprocessorDefines);
 
-		StringVector::const_iterator i;
+		for(auto& arg : args)
+		{
+			if(!StringUtil::startsWith(arg, "-"))
+				arg = "-D" + arg;
+		}
+
+		StringVector::iterator i;
 		if (mSelectedCgProfile == CG_PROFILE_VS_1_1)
 		{
 			// Need the 'dcls' argument whenever we use this profile
@@ -1193,6 +1200,9 @@ namespace Ogre {
 			dict->addParameter(ParameterDef("profiles",
 				"Space-separated list of Cg profiles supported by this profile.",
 				PT_STRING),&msCmdProfiles);
+			dict->addParameter(ParameterDef("target",
+				"Space-separated list of Cg profiles supported by this profile.",
+				PT_STRING),&msCmdProfiles);
 			dict->addParameter(ParameterDef("compile_arguments",
 				"A string of compilation arguments to pass to the Cg compiler.",
 				PT_STRING),&msCmdArgs);
