@@ -290,6 +290,14 @@ namespace Ogre
         case VET_FLOAT4:
             return DXGI_FORMAT_R32G32B32A32_FLOAT;
 
+        // Float16
+        case VET_HALF1:
+            return DXGI_FORMAT_R16_FLOAT;
+        case VET_HALF2:
+            return DXGI_FORMAT_R16G16_FLOAT;
+        case VET_HALF4:
+            return DXGI_FORMAT_R16G16B16A16_FLOAT;
+
         // Signed short
         case VET_SHORT1:
             return DXGI_FORMAT_R16_SINT;
@@ -347,33 +355,6 @@ namespace Ogre
         return DXGI_FORMAT_R32G32B32_FLOAT;
     }
     //---------------------------------------------------------------------
-    VertexElementSemantic D3D11Mappings::get(LPCSTR sem)
-    {
-        // todo - add to ogre - POSITIONT and PSIZE ("Transformed vertex position" and "Point size")
-
-        if( strcmp(sem, "BLENDINDICES") == 0 )
-            return VES_BLEND_INDICES;
-        if( strcmp(sem, "BLENDWEIGHT") == 0 )
-            return VES_BLEND_WEIGHTS;
-        if( strcmp(sem, "COLOR") == 0 )
-            return VES_DIFFUSE;
-//      if( strcmp(sem, "COLOR") == 0 )
-//          return VES_SPECULAR;
-        if( strcmp(sem, "NORMAL") == 0 )
-            return VES_NORMAL;
-        if( strcmp(sem, "POSITION") == 0 )
-            return VES_POSITION;
-        if( strcmp(sem, "TEXCOORD") == 0 )
-            return VES_TEXTURE_COORDINATES;
-        if( strcmp(sem, "BINORMAL") == 0 )
-            return VES_BINORMAL;
-        if( strcmp(sem, "TANGENT") == 0 )
-            return VES_TANGENT;
-
-        // to keep compiler happy
-        return VES_POSITION;
-    }
-    //---------------------------------------------------------------------
     LPCSTR D3D11Mappings::get(VertexElementSemantic sem)
     {
         // todo - add to ogre - POSITIONT and PSIZE ("Transformed vertex position" and "Point size")
@@ -745,7 +726,7 @@ namespace Ogre
 		}
 
         UINT retVal = 0;
-        if( !(usage & TU_NOT_SRV) )
+        if( !(usage & TU_NOT_SAMPLED) )
             retVal |= D3D11_BIND_SHADER_RESOURCE;
 
         if( isRenderTarget )
@@ -764,8 +745,8 @@ namespace Ogre
 
         UINT flags = 0;
 
-		if((bindflags & D3D11_BIND_SHADER_RESOURCE) && (bindflags & D3D11_BIND_RENDER_TARGET))
-			flags |= D3D11_RESOURCE_MISC_GENERATE_MIPS;
+	    if((bindflags & D3D11_BIND_SHADER_RESOURCE) && (bindflags & D3D11_BIND_RENDER_TARGET) && (usage & TU_AUTOMIPMAP))
+		    flags |= D3D11_RESOURCE_MISC_GENERATE_MIPS;
 
         if(textype == TEX_TYPE_CUBE_MAP)
             flags |= D3D11_RESOURCE_MISC_TEXTURECUBE;
